{-----------------------------------------------------------------------------}
{ In diesem Modul ist der Parser zur numerischen Auswertung eines Strings     }
{ enthalten. Die Idee fr den rekursiven Aufbau stammt aus dem ST-Magazin     }
{ 02/87 Seite 63ff.                                                           }
{                                                                             }
{ (c) WURMSOFTWARE im Mai 1987                                                }
{-----------------------------------------------------------------------------}

{====================    Hilfsproceduren     =================================}

{---- Do_Init_Error : Setzt Fehlertexte --------------------------------------}

PROCEDURE Do_Init_Error ;

 BEGIN
  error_str[1]  := ' Potenz einer negativen Zahl' ;
  error_str[2]  := ' TAN nicht definiert fr n*pi/2' ;
  error_str[3]  := ' Argument der ARC-Funktionen nicht aus (-1,1)' ;
  error_str[4]  := ' AR- Funktionen mit verbotenem Argument' ;
  error_str[5]  := ' LOG aus einer neagtiven Zahl' ;
  error_str[6]  := ' Wurzel aus einer negativen Zahl' ;
  error_str[7]  := ' LN aus einer negativen Zahl' ;
  error_str[8]  := ' Division durch null' ;
  error_str[9]  := ' Overflow bei EXP' ;
  error_str[10] := ' Overflow bei Multiplikation' ;
 END ;


{==================== zustzliche Funktionen =================================}

FUNCTION NPwrOfTen (zahl : integer) : real ;

 VAR  zwischen           : real ;

 BEGIN
  zwischen := PwrOfTen (ABS(zahl)) ;
  IF zahl < 0 THEN zwischen := 1 / zwischen ;
  NPwrOfTen := zwischen ;
 END ;

FUNCTION Potenz (basis, exponent : real) : real ;

 BEGIN
  Potenz := EXP (exponent * LN (basis)) ;
 END ;

FUNCTION Tan (arg : real) : real ;

 BEGIN
  tan := sin(arg) / cos (arg) ;
 END ;

FUNCTION arcsin (arg : real) : real ;

 BEGIN
  arcsin := arctan (arg/(sqrt(1-arg*arg))) ;
 END ;

FUNCTION arccos (arg : real) : real ;

 BEGIN
  arccos := 1 / (arctan (arg/(sqrt(1-arg*arg)))) ;
 END ;

FUNCTION sinh (arg : real) : real ;

 BEGIN
  sinh := 0.5 * (exp(arg)-exp(-arg)) ;
 END ;

FUNCTION cosh (arg : real) : real ;

 BEGIN
  cosh := 0.5 * (exp(arg)+exp(-arg)) ;
 END ;

FUNCTION tanh (arg : real) : real ;

 BEGIN
  tanh := sinh(arg) / cosh(arg) ;
 END ;

FUNCTION arsinh (arg : real) : real ;

 BEGIN
  arsinh := ln (arg + sqrt (arg*arg + 1)) ;
 END ;

FUNCTION arcosh (arg : real) : real ;

 BEGIN
  arcosh := ln (arg - sqrt (arg*arg - 1)) ;
 END ;

FUNCTION artanh (arg : real) : real ;

 BEGIN
  artanh := 0.5 * ln ((1+arg)/(1-arg)) ;
 END ;

FUNCTION Fak (arg : real) : real ;

 VAR i            : integer ;
     help         : real ;

 BEGIN
  help := 1 ;
  FOR i := 1 TO TRUNC (arg) DO help := help * i ;
  fak := help ;
 END ;

FUNCTION log (arg : real) : real ;

 BEGIN
  log := ln(arg) / ln(10) ;
 END ;


{==========================  FORMELANALYSE  ==================================}


{------- Fkt_Analyse : Untersucht Funktion auf Konstantennamen ---------------}

FUNCTION Fkt_Analyse (VAR fkt           : str255 ; VAR konst : konst_type ;
                           var_x, var_y : str255) : boolean ;

 VAR  dummy_str          : str255 ;
      pos, index, dummy,
      lauf               : integer ;
      zei                : char ;
      erfolg, richtig    : boolean ;

 FUNCTION  Next_zei (VAR position : integer) : char ;

  VAR   help           : char ;

  BEGIN
   REPEAT
    position := position + 1 ;
    IF position <= LENGTH (fkt) THEN help := fkt[position]
    ELSE                             help := '' ;
   UNTIL help <> ' ' ;
   Next_zei := help ;
  END ;


 PROCEDURE Zeichen_Wandel (VAR zeile : str255) ;

  VAR   lauf              : integer ;

  BEGIN
   FOR lauf := 1 TO LENGTH (zeile) DO
    BEGIN
     IF zeile[lauf] IN ['A'..'Z'] THEN
      zeile[lauf] := CHR (ORD (zeile[lauf]) + (ORD('a')-ORD('A'))) ;
    END ;
  END ;


 {** Beginn der Procedure **}

 BEGIN
  richtig := true ;
  pos := 0 ; index := 1 ;
  WHILE pos < LENGTH (fkt) DO
   BEGIN
    zei := Next_zei (pos) ;
    IF zei IN ['a'..'z','A'..'Z'] THEN
     BEGIN
      dummy := pos ;
      REPEAT
       zei := Next_zei (pos) ;
      UNTIL NOT (zei IN ['a'..'z','A'..'Z']) ;
      dummy_str := COPY (fkt, dummy, pos-dummy) ;
      Zeichen_Wandel (dummy_str) ;
      IF (dummy_str <> 'sin'   ) AND (dummy_str <> 'cos'   ) AND
         (dummy_str <> 'tan'   ) AND (dummy_str <> 'arcsin') AND
         (dummy_str <> 'arccos') AND (dummy_str <> 'arctan') AND
         (dummy_str <> 'sinh'  ) AND (dummy_str <> 'cosh'  ) AND
         (dummy_str <> 'tanh'  ) AND (dummy_str <> 'arsinh') AND
         (dummy_str <> 'arcosh') AND (dummy_str <> 'artanh') AND
         (dummy_str <> 'ln'    ) AND (dummy_str <> 'log'   ) AND
         (dummy_str <> 'exp'   ) AND (dummy_str <> 'sqrt'  ) AND
         (dummy_str <> 'frac'  ) AND (dummy_str <> 'int'   ) AND
         (dummy_str <> 'e'     ) AND (dummy_str <> 'E'     ) AND
         (dummy_str <> 'abs'   ) AND (dummy_str <> var_x)    AND
         (dummy_str <> var_y)    THEN

       BEGIN
        erfolg := false ;
        FOR lauf := 1 TO (index-1) DO
         IF konst[lauf].name = dummy_str THEN erfolg := true ;
        IF NOT erfolg THEN
         BEGIN
          IF index > 20 THEN
           BEGIN
            button := Do_Alert
            ('[3][Nicht mehr als 20 Konstanten | benutzen ][ verstanden ]', 1);
            richtig := false ;
           END
          ELSE
           BEGIN
            konst[index].name := dummy_str ;
            index := index + 1 ;
           END ;
         END ;
       END ;
     END ;
   END ;
  konst[index].name := 'ende' ;
 END ;


{------- Zahl_Auswerten : Wandelt Str255 in Realzahl um ----------------------}

FUNCTION Zahl_Auswerten (zahl_str : str255 ; VAR ergebnis : real) : boolean ;

 VAR
  neg_flag, pot_neg_flag, exp_flag   : boolean ;
  lauf, zeiger, potenz               : integer ;

 BEGIN
  neg_flag := false ; pot_neg_flag := false ;
  exp_flag := false ;
  lauf := 1 ; ergebnis := 0 ; potenz := 0 ;
  zeiger := 1 ;
  IF zahl_str[zeiger] = '-' THEN
   BEGIN
    neg_flag := true ;
    zeiger := zeiger + 1 ;
   END ;
  WHILE  (zeiger <= LENGTH(zahl_str))  AND
         NOT (zahl_str[zeiger] IN ['.','+','-','E','e']) DO
   BEGIN
    ergebnis := ergebnis * 10 + ORD (zahl_str[zeiger]) - 48 ;
    zeiger := zeiger + 1 ;
   END ;
  IF zahl_str[zeiger] = '.' THEN
   BEGIN
    zeiger := zeiger + 1 ;
     WHILE (zeiger <= LENGTH(zahl_str)) AND
           NOT (zahl_str[zeiger] IN ['+','-','E','e']) DO
      BEGIN
       ergebnis :=
       ergebnis + (ORD (zahl_str[zeiger]) - 48)*(1/PwrOfTen(lauf)) ;
       zeiger := zeiger + 1 ;
       lauf := lauf + 1 ;
      END ;
     END ;
  IF (zahl_str[zeiger] = 'E') OR (zahl_str[zeiger] = 'e') THEN
   zeiger := zeiger + 1 ;
  IF zahl_str[zeiger] = '+' THEN zeiger := zeiger + 1 ;
  IF zahl_str[zeiger] = '-' THEN
   BEGIN
    pot_neg_flag := true ;
    zeiger := zeiger + 1 ;
   END ;
  WHILE  zeiger <= LENGTH(zahl_str) DO
   BEGIN
    exp_flag := true ;
    potenz := potenz * 10 + ORD (zahl_str[zeiger]) - 48 ;
    zeiger := zeiger + 1 ;
   END ;

  Zahl_Auswerten := true ;
  IF exp_flag THEN
   BEGIN
    IF ABS (potenz) < 18 THEN
     BEGIN
      IF  pot_neg_flag THEN ergebnis := ergebnis * (1 / PwrOfTen (potenz))
      ELSE                  ergebnis := ergebnis * PwrOfTen (potenz) ;
     END
    ELSE
     Zahl_Auswerten := false ;
   END ;
   IF neg_flag THEN ergebnis := -1 * ergebnis ;
  END ;


{---- Do_Berechnen : Berechnet den Wert der Funktion -------------------------}

FUNCTION  Do_Berechnen (    fkt_str : str255 ; konst         : konst_type  ;
                            x, y    : real   ; var_x, var_y  : str255  ;
                        VAR wert    : real   )               : integer ;

 CONST zei_end   = '#' ;

 TYPE  f_array_type    = ARRAY [1..20] OF str255 ;

 VAR   zei                    : char ;
       f_array                : f_array_type ;
       pos, f_max, fehler     : integer ;

 FUNCTION  Ausdruck : real ; FORWARD ;

 FUNCTION  Faktor   : real ; FORWARD ;


 PROCEDURE Init_Funktionen ;

  BEGIN
   f_max := 20 ;
   f_array [ 1]  := 'arsinh' ;
   f_array [ 2]  := 'arcosh' ;
   f_array [ 3]  := 'artanh' ;
   f_array [ 4]  := 'arcsin' ;
   f_array [ 5]  := 'arccos' ;
   f_array [ 6]  := 'arctan' ;
   f_array [ 7]  := 'sinh' ;
   f_array [ 8]  := 'cosh' ;
   f_array [ 9]  := 'tanh' ;
   f_array [10]  := 'sqrt' ;
   f_array [11]  := 'frac' ;
   f_array [12]  := 'int' ;
   f_array [13]  := 'abs' ;
   f_array [14]  := 'sin' ;
   f_array [15]  := 'cos' ;
   f_array [16]  := 'tan' ;
   f_array [17]  := 'log' ;
   f_array [18]  := 'exp' ;
   f_array [19]  := 'ln' ;
   f_array [20]  := '!' ;
  END ;


 PROCEDURE Zeichen_Wandel (VAR zeile : str255) ;

  VAR   lauf              : integer ;

  BEGIN
   FOR lauf := 1 TO LENGTH (zeile) DO
    BEGIN
     IF zeile[lauf] IN ['A'..'Z'] THEN
      zeile[lauf] := CHR (ORD (zeile[lauf]) + (ORD('a')-ORD('A'))) ;
    END ;
  END ;


 FUNCTION  Next_zei (VAR position : integer) : char ;

  VAR   help           : char ;

  BEGIN
   REPEAT
    position := position + 1 ;
    IF position <= LENGTH (fkt_str) THEN help := fkt_str[position]
    ELSE                             help := zei_end ;
   UNTIL help <> ' ' ;
   Next_zei := help ;
  END ;


 PROCEDURE Zahl_Bastel (VAR wert : real) ;

  VAR  anfang                    : integer ;
       zahl_str                  : str255 ;
       richtig                   : boolean ;

  BEGIN
   anfang := pos ;
   REPEAT
    zei := Next_zei (pos) ;
   UNTIL NOT (zei IN ['.','0'..'9']) ;
   IF zei IN ['E','e'] THEN
    BEGIN
     IF zei IN ['+','-'] THEN zei := Next_Zei (pos) ;
     REPEAT
      zei := Next_zei (pos) ;
     UNTIL NOT (zei IN ['0'..'9']) ;
    END ;
   zahl_str := COPY (fkt_str, anfang, pos-anfang) ;
   richtig  := Zahl_Auswerten (zahl_str, wert) ;
   IF NOT richtig THEN fehler := pos ;
  END ;


 FUNCTION  Konstanten : real ;

  VAR  index, lang    : integer ;
       erfolg         : boolean ;
       test_str       : str255 ;

  BEGIN
   index := 1 ; erfolg := false ;
   WHILE (konst[index].name <> 'ende') AND (erfolg = false) DO
    BEGIN
     lang := LENGTH (konst[index].name) ;
     IF (pos+lang-1) <= LENGTH (fkt_str) THEN
      test_str := COPY (fkt_str, pos, lang)
     ELSE
      test_str := 'nichts !!' ;
     IF test_str = konst[index].name THEN
      BEGIN
       Konstanten := konst[index].wert ;
       erfolg := true ;
       pos := pos + lang - 1 ;
       zei  := Next_zei(pos) ;
      END ;
     index := index + 1 ;
    END ;

   IF NOT erfolg THEN
    BEGIN
     lang := LENGTH (var_x) ;
     IF (pos+lang-1) <= LENGTH (fkt_str) THEN
      test_str := COPY (fkt_str, pos, lang)
     ELSE
      test_str := 'nichts !!' ;
     IF test_str = var_x THEN
      BEGIN
       Konstanten := x ;
       erfolg := true ;
       pos := pos + lang - 1 ;
       zei  := Next_zei(pos) ;
      END ;
    END ;

   IF NOT erfolg THEN
    BEGIN
     lang := LENGTH (var_y) ;
     IF (pos+lang-1) <= LENGTH (fkt_str) THEN
      test_str := COPY (fkt_str, pos, lang)
     ELSE
      test_str := 'nichts !!' ;
     IF test_str = var_y THEN
      BEGIN
       Konstanten := y ;
       erfolg := true ;
       pos := pos + lang - 1 ;
       zei  := Next_zei(pos) ;
      END ;
    END ;

   IF NOT erfolg THEN fehler := pos ;
  END ;


 FUNCTION  Stand_Fkt : real ;

  VAR  test_str            : str255 ;
       zw_y                : real ;
       index, lang         : integer ;
       erfolg              : boolean ;

  BEGIN
   erfolg := false ;
   FOR index := 1 TO f_max DO
    BEGIN
     IF erfolg = false THEN
      BEGIN
       lang := LENGTH (f_array[index]) ;
       IF (pos+lang) <= LENGTH (fkt_str) THEN
        test_str := COPY (fkt_str, pos, lang)
       ELSE
        test_str := 'hier ist nichts !!' ;
       Zeichen_Wandel (test_str) ;
       IF test_str = f_array[index] THEN
        BEGIN
         pos  := pos + lang - 1;
         zei  := Next_zei(pos) ;
         zw_y := Faktor ;
         CASE index OF
          1   :                           Stand_Fkt := ARSINH (zw_y) ;
          2   : IF ABS(zw_y) >= 1 THEN    Stand_Fkt := ARCOSH (zw_y)
                ELSE                      fehler    := -4 ;
          3   : IF ABS(zw_y) <  1 THEN    Stand_Fkt := ARTANH (zw_y)
                ELSE                      fehler    := -4 ;
          4   : IF ABS(zw_y) <  1 THEN    Stand_Fkt := ARCSIN (zw_y)
                ELSE                      fehler    := -3 ;
          5   : IF ABS(zw_y) <  1 THEN    Stand_Fkt := ARCCOS (zw_y)
                ELSE                      fehler    := -3 ;
          6   :                           Stand_Fkt := ARCTAN (zw_y) ;
          7   :                           Stand_Fkt := SINH (zw_y) ;
          8   :                           Stand_Fkt := COSH (zw_y) ;
          9   :                           Stand_Fkt := TANH (zw_y) ;
          10  : IF zw_y >= 0      THEN    Stand_Fkt := SQRT (zw_y)
                ELSE                      fehler    := -6 ;
          11  :                           Stand_Fkt := zw_y - TRUNC (zw_y) ;
          12  :                           Stand_Fkt := TRUNC (zw_y) ;
          13  :                           Stand_Fkt := ABS (zw_y) ;
          14  :                           Stand_Fkt := SIN (zw_y) ;
          15  :                           Stand_Fkt := COS (zw_y) ;
          16  : IF COS(zw_y) <> 0 THEN    Stand_Fkt := TAN (zw_y)
                ELSE                      fehler    := -2 ;
          17  : IF zw_y > 0       THEN    Stand_Fkt := LOG (zw_y)
                ELSE                      fehler    := -5 ;
          18  : IF zw_y < 41      THEN    Stand_Fkt := EXP (zw_y)
                ELSE                      fehler    := -9 ;
          19  : IF zw_y > 0       THEN    Stand_Fkt := LN (zw_y)
                ELSE                      fehler    := -7 ;
          20  : IF zw_y <= 20     THEN    Stand_Fkt := FAK (zw_y)
                ELSE                      fehler    := -10 ;
         END ;
         erfolg := true ;
        END ;
      END ;
    END ;
   IF NOT erfolg THEN Stand_Fkt := Konstanten ;
  END ;


 FUNCTION  Faktor ;

  VAR   zw_y             : real ;

  BEGIN
   IF zei IN ['.','0'..'9'] THEN
    Zahl_Bastel (zw_y) ;

   IF zei = '(' THEN
    BEGIN
     zei := Next_zei (pos) ;
     zw_y := Ausdruck ;
     IF zei = ')' THEN zei    := Next_zei (pos)
     ELSE              fehler := pos ;
    END ;

   IF zei IN ['!','a'..'z','A'..'Z'] THEN zw_y := Stand_Fkt ;

   Faktor := zw_y ;
  END ;


 FUNCTION  Vorzeichen_Faktor : real ;

  BEGIN
   IF zei = '-' THEN BEGIN
                      zei := Next_zei (pos) ;
                      Vorzeichen_Faktor := -1 * Faktor ;
                     END
   ELSE              Vorzeichen_Faktor := Faktor ;
  END ;


 FUNCTION  Term : real ;

  VAR   zw_y        : real ;

  BEGIN
   zw_y := Vorzeichen_Faktor ;
   WHILE zei = '^' DO
    BEGIN
     zei := Next_zei (pos) ;
     IF zw_y > 0 THEN zw_y   := Potenz (zw_y, Vorzeichen_Faktor)
     ELSE             fehler := -1 ;
    END ;
   Term := zw_y ;
  END ;


 FUNCTION  Einfach_Ausdruck : real ;

  VAR   operator            : char ;
        zw_y, teiler, multi : real ;

  BEGIN
   zw_y := Term ;
   WHILE (zei = '*') OR (zei = '/') DO
    BEGIN
     operator := zei ;
     zei := Next_zei (pos) ;
     IF operator = '*' THEN
      BEGIN
       multi := Term ;
       IF multi <> 0 THEN
        BEGIN
         IF (1e18/ABS(multi)) > ABS(zw_y) THEN zw_y   := zw_y * multi
         ELSE                                  fehler := -10 ;
        END
       ELSE
        zw_y := 0 ;     { bei Multiplikation mit 0 }
      END ;
     IF operator = '/' THEN
      BEGIN
       teiler := Term ;
       IF teiler <> 0 THEN zw_y   := zw_y / teiler
       ELSE                fehler := -8 ;
      END ;
    END ;
   Einfach_Ausdruck := zw_y ;
  END ;


 FUNCTION  Ausdruck ;

  VAR   operator    : char ;
        zw_y        : real ;

  BEGIN
   zw_y := Einfach_Ausdruck ;
   WHILE (zei = '+') OR (zei = '-') DO
    BEGIN
     operator := zei ;
     zei := Next_zei(pos) ;
     IF operator = '+' THEN zw_y := zw_y + Einfach_Ausdruck ;
     IF operator = '-' THEN zw_y := zw_y - Einfach_Ausdruck ;
    END ;
   Ausdruck := zw_y ;
  END ;


 {** Procedure zu Do_Berechnen **}

 BEGIN
  pos  := 0 ; fehler := 0 ;
  Init_Funktionen ;
  zei  := Next_Zei (pos) ;
  wert := Ausdruck ;
  IF fehler < 0 THEN wert := 0 ;
  IF zei <> zei_end THEN fehler := pos ;
  Do_Berechnen := fehler ;
 END ;


{=============================== Ende  =======================================}


